@@ -11,11 +11,7 @@ from genapi import \
 h_template = r"""
 #ifdef _UMATHMODULE
 
-#ifdef NPY_ENABLE_SEPARATE_COMPILATION
 extern NPY_NO_EXPORT PyTypeObject PyUFunc_Type;
-#else
-NPY_NO_EXPORT PyTypeObject PyUFunc_Type;
-#endif
 
 %s
 
@@ -37,7 +33,7 @@ static void **PyUFunc_API=NULL;
 
 %s
 
-static int
+static NPY_INLINE int
 _import_umath(void)
 {
   PyObject *numpy = PyImport_ImportModule("numpy.core.umath");
@@ -172,11 +168,13 @@ def do_generate_api(targets, sources):
     api_name = 'PyUFunc_API'
     for f in ufunc_api_list:
         name = f.name
-        index = ufunc_api_index[name]
-        ufunc_api_dict[name] = FunctionApi(f.name, index, f.return_type,
-                                           f.args, api_name)
+        index = ufunc_api_index[name][0]
+        annotations = ufunc_api_index[name][1:]
+        ufunc_api_dict[name] = FunctionApi(f.name, index, annotations,
+                                           f.return_type, f.args, api_name)
 
-    for name, index in numpy_api.ufunc_types_api.items():
+    for name, val in numpy_api.ufunc_types_api.items():
+        index = val[0]
         ufunc_api_dict[name] = TypeApi(name, index, 'PyTypeObject', api_name)
 
     # set up object API
