@@ -686,7 +686,7 @@ add_newdoc('numpy.core.umath', 'ceil',
 
     Returns
     -------
-    y : {ndarray, scalar}
+    y : ndarray or scalar
         The ceiling of each element in `x`, with `float` dtype.
 
     See Also
@@ -716,7 +716,7 @@ add_newdoc('numpy.core.umath', 'trunc',
 
     Returns
     -------
-    y : {ndarray, scalar}
+    y : ndarray or scalar
         The truncated value of each element in `x`.
 
     See Also
@@ -931,9 +931,9 @@ add_newdoc('numpy.core.umath', 'divide',
 
     Returns
     -------
-    y : {ndarray, scalar}
-        The quotient `x1/x2`, element-wise. Returns a scalar if
-        both  `x1` and `x2` are scalars.
+    y : ndarray or scalar
+        The quotient ``x1/x2``, element-wise. Returns a scalar if
+        both ``x1`` and ``x2`` are scalars.
 
     See Also
     --------
@@ -942,13 +942,13 @@ add_newdoc('numpy.core.umath', 'divide',
 
     Notes
     -----
-    Equivalent to `x1` / `x2` in terms of array-broadcasting.
+    Equivalent to ``x1`` / ``x2`` in terms of array-broadcasting.
 
-    Behavior on division by zero can be changed using `seterr`.
+    Behavior on division by zero can be changed using ``seterr``.
 
-    When both `x1` and `x2` are of an integer type, `divide` will return
-    integers and throw away the fractional part. Moreover, division by zero
-    always yields zero in integer arithmetic.
+    In Python 2, when both ``x1`` and ``x2`` are of an integer type,
+    ``divide`` will behave like ``floor_divide``. In Python 3, it behaves
+    like ``true_divide``.
 
     Examples
     --------
@@ -961,20 +961,20 @@ add_newdoc('numpy.core.umath', 'divide',
            [ Inf,  4. ,  2.5],
            [ Inf,  7. ,  4. ]])
 
-    Note the behavior with integer types:
+    Note the behavior with integer types (Python 2 only):
 
     >>> np.divide(2, 4)
     0
     >>> np.divide(2, 4.)
     0.5
 
-    Division by zero always yields zero in integer arithmetic, and does not
-    raise an exception or a warning:
+    Division by zero always yields zero in integer arithmetic (again,
+    Python 2 only), and does not raise an exception or a warning:
 
     >>> np.divide(np.array([0, 1], dtype=int), np.array([0, 0], dtype=int))
     array([0, 0])
 
-    Division by zero can, however, be caught using `seterr`:
+    Division by zero can, however, be caught using ``seterr``:
 
     >>> old_err_state = np.seterr(divide='raise')
     >>> np.divide(1, 0)
@@ -999,7 +999,7 @@ add_newdoc('numpy.core.umath', 'equal',
 
     Returns
     -------
-    out : {ndarray, bool}
+    out : ndarray or bool
         Output array of bools, or a single bool if x1 and x2 are scalars.
 
     See Also
@@ -1172,7 +1172,7 @@ add_newdoc('numpy.core.umath', 'fabs',
 
     Returns
     -------
-    y : {ndarray, scalar}
+    y : ndarray or scalar
         The absolute values of `x`, the returned values are always floats.
 
     See Also
@@ -1202,7 +1202,7 @@ add_newdoc('numpy.core.umath', 'floor',
 
     Returns
     -------
-    y : {ndarray, scalar}
+    y : ndarray or scalar
         The floor of each element in `x`.
 
     See Also
@@ -1225,8 +1225,10 @@ add_newdoc('numpy.core.umath', 'floor',
 
 add_newdoc('numpy.core.umath', 'floor_divide',
     """
-    Return the largest integer smaller or equal to the division of the
-    inputs.
+    Return the largest integer smaller or equal to the division of the inputs.
+    It is equivalent to the Python ``//`` operator and pairs with the
+    Python ``%`` (`remainder`), function so that ``b = a % b + b * (a // b)``
+    up to roundoff.
 
     Parameters
     ----------
@@ -1243,6 +1245,7 @@ add_newdoc('numpy.core.umath', 'floor_divide',
 
     See Also
     --------
+    remainder : Remainder complementary to floor_divide.
     divide : Standard division.
     floor : Round a number to the nearest integer toward minus infinity.
     ceil : Round a number to the nearest integer toward infinity.
@@ -1632,7 +1635,7 @@ add_newdoc('numpy.core.umath', 'isnan',
 
     Returns
     -------
-    y : {ndarray, bool}
+    y : ndarray or bool
         For scalar input, the result is a new boolean with value True if
         the input is NaN; otherwise the value is False.
 
@@ -2046,7 +2049,7 @@ add_newdoc('numpy.core.umath', 'logical_and',
 
     Returns
     -------
-    y : {ndarray, bool}
+    y : ndarray or bool
         Boolean result with the same shape as `x1` and `x2` of the logical
         AND operation on corresponding elements of `x1` and `x2`.
 
@@ -2112,7 +2115,7 @@ add_newdoc('numpy.core.umath', 'logical_or',
 
     Returns
     -------
-    y : {ndarray, bool}
+    y : ndarray or bool
         Boolean result with the same shape as `x1` and `x2` of the logical
         OR operation on elements of `x1` and `x2`.
 
@@ -2193,7 +2196,7 @@ add_newdoc('numpy.core.umath', 'maximum',
 
     Returns
     -------
-    y : {ndarray, scalar}
+    y : ndarray or scalar
         The maximum of `x1` and `x2`, element-wise.  Returns scalar if
         both  `x1` and `x2` are scalars.
 
@@ -2251,7 +2254,7 @@ add_newdoc('numpy.core.umath', 'minimum',
 
     Returns
     -------
-    y : {ndarray, scalar}
+    y : ndarray or scalar
         The minimum of `x1` and `x2`, element-wise.  Returns scalar if
         both  `x1` and `x2` are scalars.
 
@@ -2309,8 +2312,8 @@ add_newdoc('numpy.core.umath', 'fmax',
 
     Returns
     -------
-    y : {ndarray, scalar}
-        The minimum of `x1` and `x2`, element-wise.  Returns scalar if
+    y : ndarray or scalar
+        The maximum of `x1` and `x2`, element-wise.  Returns scalar if
         both  `x1` and `x2` are scalars.
 
     See Also
@@ -2366,7 +2369,7 @@ add_newdoc('numpy.core.umath', 'fmin',
 
     Returns
     -------
-    y : {ndarray, scalar}
+    y : ndarray or scalar
         The minimum of `x1` and `x2`, element-wise.  Returns scalar if
         both  `x1` and `x2` are scalars.
 
@@ -2689,9 +2692,9 @@ add_newdoc('numpy.core.umath', 'remainder',
     """
     Return element-wise remainder of division.
 
-    Computes ``x1 - floor(x1 / x2) * x2``, the result has the same sign as
-    the divisor `x2`. It is equivalent to the Python modulus operator
-    ``x1 % x2`` and should not be confused with the Matlab(TM) ``rem``
+    Computes the remainder complementary to the `floor_divide` function.  It is
+    equivalent to the Python modulus operator``x1 % x2`` and has the same sign
+    as the divisor `x2`. It should not be confused with the Matlab(TM) ``rem``
     function.
 
     Parameters
@@ -2707,11 +2710,12 @@ add_newdoc('numpy.core.umath', 'remainder',
     Returns
     -------
     y : ndarray
-        The remainder of the quotient ``x1/x2``, element-wise. Returns a
-        scalar if both  `x1` and `x2` are scalars.
+        The element-wise remainder of the quotient ``floor_divide(x1, x2)``.
+        Returns a scalar if both  `x1` and `x2` are scalars.
 
     See Also
     --------
+    floor_divide : Equivalent of Python ``//`` operator.
     fmod : Equivalent of the Matlab(TM) ``rem`` function.
     divide, floor
 
@@ -2780,7 +2784,7 @@ add_newdoc('numpy.core.umath', 'rint',
 
     Returns
     -------
-    out : {ndarray, scalar}
+    out : ndarray or scalar
         Output array is same shape and type as `x`.
 
     See Also
@@ -2799,7 +2803,13 @@ add_newdoc('numpy.core.umath', 'sign',
     """
     Returns an element-wise indication of the sign of a number.
 
-    The `sign` function returns ``-1 if x < 0, 0 if x==0, 1 if x > 0``.
+    The `sign` function returns ``-1 if x < 0, 0 if x==0, 1 if x > 0``.  nan
+    is returned for nan inputs.
+
+    For complex inputs, the `sign` function returns
+    ``sign(x.real) + 0j if x.real != 0 else sign(x.imag) + 0j``.
+
+    complex(nan, 0) is returned for complex nan inputs.
 
     Parameters
     ----------
@@ -2811,12 +2821,20 @@ add_newdoc('numpy.core.umath', 'sign',
     y : ndarray
       The sign of `x`.
 
+    Notes
+    -----
+    There is more than one definition of sign in common use for complex
+    numbers.  The definition used here is equivalent to :math:`x/\\sqrt{x*x}`
+    which is different from a common alternative, :math:`x/|x|`.
+
     Examples
     --------
     >>> np.sign([-5., 4.5])
     array([-1.,  1.])
     >>> np.sign(0)
     0
+    >>> np.sign(5-2j)
+    (1+0j)
 
     """)
 
@@ -3104,6 +3122,35 @@ add_newdoc('numpy.core.umath', 'sqrt',
 
     """)
 
+add_newdoc('numpy.core.umath', 'cbrt',
+    """
+    Return the cube-root of an array, element-wise.
+
+    .. versionadded:: 1.10.0
+
+    Parameters
+    ----------
+    x : array_like
+        The values whose cube-roots are required.
+    out : ndarray, optional
+        Alternate array object in which to put the result; if provided, it
+        must have the same shape as `x`
+
+    Returns
+    -------
+    y : ndarray
+        An array of the same shape as `x`, containing the cube
+        cube-root of each element in `x`.
+        If `out` was provided, `y` is a reference to it.
+
+
+    Examples
+    --------
+    >>> np.cbrt([1,8,27])
+    array([ 1.,  2.,  3.])
+
+    """)
+
 add_newdoc('numpy.core.umath', 'square',
     """
     Return the element-wise square of the input.
@@ -3324,9 +3371,6 @@ add_newdoc('numpy.core.umath', 'true_divide',
 
     """)
 
-# This doc is not currently used, but has been converted to a C string
-# that can be found in numpy/core/src/umath/umathmodule.c where the
-# frexp ufunc is constructed.
 add_newdoc('numpy.core.umath', 'frexp',
     """
     Decompose the elements of x into mantissa and twos exponent.
@@ -3339,9 +3383,9 @@ add_newdoc('numpy.core.umath', 'frexp',
     ----------
     x : array_like
         Array of numbers to be decomposed.
-    out1: ndarray, optional
+    out1 : ndarray, optional
         Output array for the mantissa. Must have the same shape as `x`.
-    out2: ndarray, optional
+    out2 : ndarray, optional
         Output array for the exponent. Must have the same shape as `x`.
 
     Returns
@@ -3372,9 +3416,6 @@ add_newdoc('numpy.core.umath', 'frexp',
 
     """)
 
-# This doc is not currently used, but has been converted to a C string
-# that can be found in numpy/core/src/umath/umathmodule.c where the
-# ldexp ufunc is constructed.
 add_newdoc('numpy.core.umath', 'ldexp',
     """
     Returns x1 * 2**x2, element-wise.
