@@ -109,10 +109,12 @@ PyArray_ArgMax(PyArrayObject *op, int axis, PyArrayObject *out)
         }
     }
     else {
-        if (PyArray_SIZE(out) !=
-                PyArray_MultiplyList(PyArray_DIMS(ap), PyArray_NDIM(ap) - 1)) {
-            PyErr_SetString(PyExc_TypeError,
-                    "invalid shape for output array.");
+        if ((PyArray_NDIM(out) != PyArray_NDIM(ap) - 1) ||
+                !PyArray_CompareLists(PyArray_DIMS(out), PyArray_DIMS(ap),
+                                      PyArray_NDIM(out))) {
+            PyErr_SetString(PyExc_ValueError,
+                    "output array does not match result of np.argmax.");
+            goto fail;
         }
         rp = (PyArrayObject *)PyArray_FromArray(out,
                               PyArray_DescrFromType(NPY_INTP),
@@ -222,10 +224,12 @@ PyArray_ArgMin(PyArrayObject *op, int axis, PyArrayObject *out)
         }
     }
     else {
-        if (PyArray_SIZE(out) !=
-                PyArray_MultiplyList(PyArray_DIMS(ap), PyArray_NDIM(ap) - 1)) {
-            PyErr_SetString(PyExc_TypeError,
-                    "invalid shape for output array.");
+        if ((PyArray_NDIM(out) != PyArray_NDIM(ap) - 1) ||
+                !PyArray_CompareLists(PyArray_DIMS(out), PyArray_DIMS(ap),
+                                      PyArray_NDIM(out))) {
+            PyErr_SetString(PyExc_ValueError,
+                    "output array does not match result of np.argmin.");
+            goto fail;
         }
         rp = (PyArrayObject *)PyArray_FromArray(out,
                               PyArray_DescrFromType(NPY_INTP),
@@ -614,7 +618,7 @@ PyArray_Round(PyArrayObject *a, int decimals, PyArrayObject *out)
         }
 
         /* arr.real = a.real.round(decimals) */
-        part = PyObject_GetAttrString(arr, "real");
+        part = PyObject_GetAttrString((PyObject *)a, "real");
         if (part == NULL) {
             Py_DECREF(arr);
             return NULL;
@@ -635,7 +639,7 @@ PyArray_Round(PyArrayObject *a, int decimals, PyArrayObject *out)
         }
 
         /* arr.imag = a.imag.round(decimals) */
-        part = PyObject_GetAttrString(arr, "imag");
+        part = PyObject_GetAttrString((PyObject *)a, "imag");
         if (part == NULL) {
             Py_DECREF(arr);
             return NULL;
@@ -917,7 +921,9 @@ PyArray_Clip(PyArrayObject *self, PyObject *min, PyObject *max, PyArrayObject *o
     func = PyArray_DESCR(self)->f->fastclip;
     if (func == NULL
         || (min != NULL && !PyArray_CheckAnyScalar(min))
-        || (max != NULL && !PyArray_CheckAnyScalar(max))) {
+        || (max != NULL && !PyArray_CheckAnyScalar(max))
+        || PyArray_ISBYTESWAPPED(self)
+        || (out && PyArray_ISBYTESWAPPED(out))) {
         return _slow_array_clip(self, min, max, out);
     }
     /* Use the fast scalar clip function */
@@ -1028,7 +1034,6 @@ PyArray_Clip(PyArrayObject *self, PyObject *min, PyObject *max, PyArrayObject *o
         }
     }
 
-
     /*
      * Check to see if input is single-segment, aligned,
      * and in native byteorder
@@ -1134,12 +1139,6 @@ PyArray_Clip(PyArrayObject *self, PyObject *min, PyObject *max, PyArrayObject *o
                         "same shape as the input.");
         goto fail;
     }
-    if (PyArray_DATA(newout) != PyArray_DATA(newin)) {
-        if (PyArray_AssignArray(newout, newin,
-                    NULL, NPY_DEFAULT_ASSIGN_CASTING) < 0) {
-            goto fail;
-        }
-    }
 
     /* Now we can call the fast-clip function */
     min_data = max_data = NULL;
@@ -1178,7 +1177,7 @@ PyArray_Clip(PyArrayObject *self, PyObject *min, PyObject *max, PyArrayObject *o
 NPY_NO_EXPORT PyObject *
 PyArray_Conjugate(PyArrayObject *self, PyArrayObject *out)
 {
-    if (PyArray_ISCOMPLEX(self)) {
+    if (PyArray_ISCOMPLEX(self) || PyArray_ISOBJECT(self)) {
         if (out == NULL) {
             return PyArray_GenericUnaryFunction(self,
                                                 n_ops.conjugate);
