@@ -52,9 +52,7 @@ PyArray_GetAttrString_SuppressException(PyObject *obj, char *name)
     PyObject *res = (PyObject *)NULL;
 
     /* We do not need to check for special attributes on trivial types */
-    if (obj == Py_None ||
-            PyList_CheckExact(obj) ||
-            PyTuple_CheckExact(obj)) {
+    if (_is_basic_python_type(obj)) {
         return NULL;
     }
 
@@ -86,7 +84,7 @@ PyArray_GetAttrString_SuppressException(PyObject *obj, char *name)
 
 
 
-NPY_NO_EXPORT NPY_CASTING NPY_DEFAULT_ASSIGN_CASTING = NPY_INTERNAL_UNSAFE_CASTING_BUT_WARN_UNLESS_SAME_KIND;
+NPY_NO_EXPORT NPY_CASTING NPY_DEFAULT_ASSIGN_CASTING = NPY_SAME_KIND_CASTING;
 
 
 NPY_NO_EXPORT PyArray_Descr *
@@ -130,30 +128,6 @@ _array_find_python_scalar_type(PyObject *op)
     return NULL;
 }
 
-#if !defined(NPY_PY3K)
-static PyArray_Descr *
-_use_default_type(PyObject *op)
-{
-    int typenum, l;
-    PyObject *type;
-
-    typenum = -1;
-    l = 0;
-    type = (PyObject *)Py_TYPE(op);
-    while (l < NPY_NUMUSERTYPES) {
-        if (type == (PyObject *)(userdescrs[l]->typeobj)) {
-            typenum = l + NPY_USERDEF;
-            break;
-        }
-        l++;
-    }
-    if (typenum == -1) {
-        typenum = NPY_OBJECT;
-    }
-    return PyArray_DescrFromType(typenum);
-}
-#endif
-
 /*
  * These constants are used to signal that the recursive dtype determination in
  * PyArray_DTypeFromObject encountered a string type, and that the recursive
@@ -207,6 +181,10 @@ PyArray_DTypeFromObjectHelper(PyObject *obj, int maxdims,
     PyArray_Descr *dtype = NULL;
     PyObject *ip;
     Py_buffer buffer_view;
+    /* types for sequence handling */
+    PyObject ** objects;
+    PyObject * seq;
+    PyTypeObject * common_type;
 
     /* Check if it's an ndarray */
     if (PyArray_Check(obj)) {
@@ -488,24 +466,19 @@ PyArray_DTypeFromObjectHelper(PyObject *obj, int maxdims,
         }
     }
 
-    /* Not exactly sure what this is about... */
-#if !defined(NPY_PY3K)
-    if (PyInstance_Check(obj)) {
-        dtype = _use_default_type(obj);
-        if (dtype == NULL) {
-            goto fail;
-        }
-        else {
-            goto promote_types;
-        }
-    }
-#endif
-
     /*
      * If we reached the maximum recursion depth without hitting one
-     * of the above cases, the output dtype should be OBJECT
+     * of the above cases, and obj isn't a sequence-like object, the output
+     * dtype should be either OBJECT or a user-defined type.
+     *
+     * Note that some libraries define sequence-like classes but want them to
+     * be treated as objects, and they expect numpy to treat it as an object if
+     * __len__ is not defined.
      */
-    if (maxdims == 0 || !PySequence_Check(obj)) {
+    if (maxdims == 0 || !PySequence_Check(obj) || PySequence_Size(obj) < 0) {
+        // clear any PySequence_Size error, which corrupts further calls to it
+        PyErr_Clear();
+
         if (*out_dtype == NULL || (*out_dtype)->type_num != NPY_OBJECT) {
             Py_XDECREF(*out_dtype);
             *out_dtype = PyArray_DescrFromType(NPY_OBJECT);
@@ -516,31 +489,49 @@ PyArray_DTypeFromObjectHelper(PyObject *obj, int maxdims,
         return 0;
     }
 
-    /* Recursive case */
-    size = PySequence_Size(obj);
-    if (size < 0) {
+    /* Recursive case, first check the sequence contains only one type */
+    seq = PySequence_Fast(obj, "Could not convert object to sequence");
+    if (seq == NULL) {
         goto fail;
     }
+    size = PySequence_Fast_GET_SIZE(seq);
+    objects = PySequence_Fast_ITEMS(seq);
+    common_type = size > 0 ? Py_TYPE(objects[0]) : NULL;
+    for (i = 1; i < size; ++i) {
+        if (Py_TYPE(objects[i]) != common_type) {
+            common_type = NULL;
+            break;
+        }
+    }
+
+    /* all types are the same and scalar, one recursive call is enough */
+    if (common_type != NULL && !string_type &&
+            (common_type == &PyFloat_Type ||
+/* TODO: we could add longs if we add a range check */
+#if !defined(NPY_PY3K)
+             common_type == &PyInt_Type ||
+#endif
+             common_type == &PyBool_Type ||
+             common_type == &PyComplex_Type)) {
+        size = 1;
+    }
+
     /* Recursive call for each sequence item */
     for (i = 0; i < size; ++i) {
-        int res;
-        ip = PySequence_GetItem(obj, i);
-        if (ip == NULL) {
-            goto fail;
-        }
-        res = PyArray_DTypeFromObjectHelper(ip, maxdims - 1,
-                                            out_dtype, string_type);
+        int res = PyArray_DTypeFromObjectHelper(objects[i], maxdims - 1,
+                                                out_dtype, string_type);
         if (res < 0) {
-            Py_DECREF(ip);
+            Py_DECREF(seq);
             goto fail;
         }
         else if (res > 0) {
-            Py_DECREF(ip);
+            Py_DECREF(seq);
             return res;
         }
-        Py_DECREF(ip);
     }
 
+    Py_DECREF(seq);
+
     return 0;
 
 
@@ -609,31 +600,6 @@ _array_typedescr_fromstr(char *c_str)
     return descr;
 }
 
-NPY_NO_EXPORT int
-check_and_adjust_index(npy_intp *index, npy_intp max_item, int axis)
-{
-    /* Check that index is valid, taking into account negative indices */
-    if ((*index < -max_item) || (*index >= max_item)) {
-        /* Try to be as clear as possible about what went wrong. */
-        if (axis >= 0) {
-            PyErr_Format(PyExc_IndexError,
-                         "index %"NPY_INTP_FMT" is out of bounds "
-                         "for axis %d with size %"NPY_INTP_FMT,
-                         *index, axis, max_item);
-        } else {
-            PyErr_Format(PyExc_IndexError,
-                         "index %"NPY_INTP_FMT" is out of bounds "
-                         "for size %"NPY_INTP_FMT,
-                         *index, max_item);
-        }
-        return -1;
-    }
-    /* adjust negative indices */
-    if (*index < 0) {
-        *index += max_item;
-    }
-    return 0;
-}
 
 NPY_NO_EXPORT char *
 index2ptr(PyArrayObject *mp, npy_intp i)
@@ -645,7 +611,7 @@ index2ptr(PyArrayObject *mp, npy_intp i)
         return NULL;
     }
     dim0 = PyArray_DIMS(mp)[0];
-    if (check_and_adjust_index(&i, dim0, 0) < 0)
+    if (check_and_adjust_index(&i, dim0, 0, NULL) < 0)
         return NULL;
     if (i == 0) {
         return PyArray_DATA(mp);
@@ -675,37 +641,45 @@ _zerofill(PyArrayObject *ret)
 NPY_NO_EXPORT int
 _IsAligned(PyArrayObject *ap)
 {
-    unsigned int i, aligned = 1;
-    const unsigned int alignment = PyArray_DESCR(ap)->alignment;
+    unsigned int i;
+    npy_uintp aligned;
+    npy_uintp alignment = PyArray_DESCR(ap)->alignment;
 
-    /* The special casing for STRING and VOID types was removed
-     * in accordance with http://projects.scipy.org/numpy/ticket/1227
-     * It used to be that IsAligned always returned True for these
-     * types, which is indeed the case when they are created using
-     * PyArray_DescrConverter(), but not necessarily when using
-     * PyArray_DescrAlignConverter(). */
+    /* alignment 1 types should have a efficient alignment for copy loops */
+    if (PyArray_ISFLEXIBLE(ap) || PyArray_ISSTRING(ap)) {
+        npy_intp itemsize = PyArray_ITEMSIZE(ap);
+        /* power of two sizes may be loaded in larger moves */
+        if (((itemsize & (itemsize - 1)) == 0)) {
+            alignment = itemsize > NPY_MAX_COPY_ALIGNMENT ?
+                NPY_MAX_COPY_ALIGNMENT : itemsize;
+        }
+        else {
+            /* if not power of two it will be accessed bytewise */
+            alignment = 1;
+        }
+    }
 
     if (alignment == 1) {
         return 1;
     }
-    aligned = npy_is_aligned(PyArray_DATA(ap), alignment);
+    aligned = (npy_uintp)PyArray_DATA(ap);
 
     for (i = 0; i < PyArray_NDIM(ap); i++) {
 #if NPY_RELAXED_STRIDES_CHECKING
+        /* skip dim == 1 as it is not required to have stride 0 */
         if (PyArray_DIM(ap, i) > 1) {
             /* if shape[i] == 1, the stride is never used */
-            aligned &= npy_is_aligned((void*)PyArray_STRIDES(ap)[i],
-                                      alignment);
+            aligned |= (npy_uintp)PyArray_STRIDES(ap)[i];
         }
         else if (PyArray_DIM(ap, i) == 0) {
             /* an array with zero elements is always aligned */
             return 1;
         }
 #else /* not NPY_RELAXED_STRIDES_CHECKING */
-        aligned &= npy_is_aligned((void*)PyArray_STRIDES(ap)[i], alignment);
+        aligned |= (npy_uintp)PyArray_STRIDES(ap)[i];
 #endif /* not NPY_RELAXED_STRIDES_CHECKING */
     }
-    return aligned != 0;
+    return npy_is_aligned((void *)aligned, alignment);
 }
 
 NPY_NO_EXPORT npy_bool
@@ -754,45 +728,13 @@ _IsWriteable(PyArrayObject *ap)
 }
 
 
-/* Gets a half-open range [start, end) of offsets from the data pointer */
-NPY_NO_EXPORT void
-offset_bounds_from_strides(const int itemsize, const int nd,
-                           const npy_intp *dims, const npy_intp *strides,
-                           npy_intp *lower_offset, npy_intp *upper_offset) {
-    npy_intp max_axis_offset;
-    npy_intp lower = 0;
-    npy_intp upper = 0;
-    int i;
-
-    for (i = 0; i < nd; i++) {
-        if (dims[i] == 0) {
-            /* If the array size is zero, return an empty range */
-            *lower_offset = 0;
-            *upper_offset = 0;
-            return;
-        }
-        /* Expand either upwards or downwards depending on stride */
-        max_axis_offset = strides[i] * (dims[i] - 1);
-        if (max_axis_offset > 0) {
-            upper += max_axis_offset;
-        }
-        else {
-            lower += max_axis_offset;
-        }
-    }
-    /* Return a half-open range */
-    upper += itemsize;
-    *lower_offset = lower;
-    *upper_offset = upper;
-}
-
 /**
  * Convert an array shape to a string such as "(1, 2)".
  *
  * @param Dimensionality of the shape
  * @param npy_intp pointer to shape array
  * @param String to append after the shape `(1, 2)%s`.
- * 
+ *
  * @return Python unicode string
  */
 NPY_NO_EXPORT PyObject *
@@ -841,8 +783,110 @@ convert_shape_to_string(npy_intp n, npy_intp *vals, char *ending)
     }
     else {
         tmp = PyUString_FromFormat(")%s", ending);
-        }
+    }
     PyUString_ConcatAndDel(&ret, tmp);
     return ret;
 }
 
+
+NPY_NO_EXPORT void
+dot_alignment_error(PyArrayObject *a, int i, PyArrayObject *b, int j)
+{
+    PyObject *errmsg = NULL, *format = NULL, *fmt_args = NULL,
+             *i_obj = NULL, *j_obj = NULL,
+             *shape1 = NULL, *shape2 = NULL,
+             *shape1_i = NULL, *shape2_j = NULL;
+
+    format = PyUString_FromString("shapes %s and %s not aligned:"
+                                  " %d (dim %d) != %d (dim %d)");
+
+    shape1 = convert_shape_to_string(PyArray_NDIM(a), PyArray_DIMS(a), "");
+    shape2 = convert_shape_to_string(PyArray_NDIM(b), PyArray_DIMS(b), "");
+
+    i_obj = PyLong_FromLong(i);
+    j_obj = PyLong_FromLong(j);
+
+    shape1_i = PyLong_FromSsize_t(PyArray_DIM(a, i));
+    shape2_j = PyLong_FromSsize_t(PyArray_DIM(b, j));
+
+    if (!format || !shape1 || !shape2 || !i_obj || !j_obj ||
+            !shape1_i || !shape2_j) {
+        goto end;
+    }
+
+    fmt_args = PyTuple_Pack(6, shape1, shape2,
+                            shape1_i, i_obj, shape2_j, j_obj);
+    if (fmt_args == NULL) {
+        goto end;
+    }
+
+    errmsg = PyUString_Format(format, fmt_args);
+    if (errmsg != NULL) {
+        PyErr_SetObject(PyExc_ValueError, errmsg);
+    }
+    else {
+        PyErr_SetString(PyExc_ValueError, "shapes are not aligned");
+    }
+
+end:
+    Py_XDECREF(errmsg);
+    Py_XDECREF(fmt_args);
+    Py_XDECREF(format);
+    Py_XDECREF(i_obj);
+    Py_XDECREF(j_obj);
+    Py_XDECREF(shape1);
+    Py_XDECREF(shape2);
+    Py_XDECREF(shape1_i);
+    Py_XDECREF(shape2_j);
+}
+
+/**
+ * unpack tuple of dtype->fields (descr, offset, title[not-needed])
+ *
+ * @param "value" should be the tuple.
+ *
+ * @return "descr" will be set to the field's dtype
+ * @return "offset" will be set to the field's offset
+ *
+ * returns -1 on failure, 0 on success.
+ */
+NPY_NO_EXPORT int
+_unpack_field(PyObject *value, PyArray_Descr **descr, npy_intp *offset)
+{
+    PyObject * off;
+    if (PyTuple_GET_SIZE(value) < 2) {
+        return -1;
+    }
+    *descr = (PyArray_Descr *)PyTuple_GET_ITEM(value, 0);
+    off  = PyTuple_GET_ITEM(value, 1);
+
+    if (PyInt_Check(off)) {
+        *offset = PyInt_AsSsize_t(off);
+    }
+    else if (PyLong_Check(off)) {
+        *offset = PyLong_AsSsize_t(off);
+    }
+    else {
+        return -1;
+    }
+
+    return 0;
+}
+
+/*
+ * check whether arrays with datatype dtype might have object fields. This will
+ * only happen for structured dtypes (which may have hidden objects even if the
+ * HASOBJECT flag is false), object dtypes, or subarray dtypes whose base type
+ * is either of these.
+ */
+NPY_NO_EXPORT int
+_may_have_objects(PyArray_Descr *dtype)
+{
+    PyArray_Descr *base = dtype;
+    if (PyDataType_HASSUBARRAY(dtype)) {
+        base = dtype->subarray->base;
+    }
+
+    return (PyDataType_HASFIELDS(base) ||
+            PyDataType_FLAGCHK(base, NPY_ITEM_HASOBJECT) );
+}
