@@ -4,18 +4,25 @@
 
 #define NPY_NO_DEPRECATED_API NPY_API_VERSION
 #define _MULTIARRAYMODULE
+
 #include "numpy/arrayobject.h"
 #include "numpy/arrayscalars.h"
 #include "numpy/arrayobject.h"
 
 #include "npy_config.h"
 #include "npy_pycompat.h"
+#include "npy_import.h"
 
 #include "common.h"
 #include "arraytypes.h"
 
 #include "conversion_utils.h"
 
+static int
+PyArray_PyIntAsInt_ErrMsg(PyObject *o, const char * msg) NPY_GCC_NONNULL(2);
+static npy_intp
+PyArray_PyIntAsIntp_ErrMsg(PyObject *o, const char * msg) NPY_GCC_NONNULL(2);
+
 /****************************************************************
 * Useful function for conversion when used with PyArg_ParseTuple
 ****************************************************************/
@@ -111,7 +118,7 @@ PyArray_IntpConverter(PyObject *obj, PyArray_Dims *seq)
     }
     if (len > NPY_MAXDIMS) {
         PyErr_Format(PyExc_ValueError, "sequence too large; "
-                     "must be smaller than %d", NPY_MAXDIMS);
+                     "cannot be greater than %d", NPY_MAXDIMS);
         return NPY_FAIL;
     }
     if (len > 0) {
@@ -215,8 +222,9 @@ PyArray_AxisConverter(PyObject *obj, int *axis)
         *axis = NPY_MAXDIMS;
     }
     else {
-        *axis = PyArray_PyIntAsInt(obj);
-        if (PyErr_Occurred()) {
+        *axis = PyArray_PyIntAsInt_ErrMsg(obj,
+                               "an integer is required for the axis");
+        if (error_converting(*axis)) {
             return NPY_FAIL;
         }
     }
@@ -251,7 +259,8 @@ PyArray_ConvertMultiAxis(PyObject *axis_in, int ndim, npy_bool *out_axis_flags)
         }
         for (i = 0; i < naxes; ++i) {
             PyObject *tmp = PyTuple_GET_ITEM(axis_in, i);
-            int axis = PyArray_PyIntAsInt(tmp);
+            int axis = PyArray_PyIntAsInt_ErrMsg(tmp,
+                          "integers are required for the axis tuple elements");
             int axis_orig = axis;
             if (error_converting(axis)) {
                 return NPY_FAIL;
@@ -281,7 +290,8 @@ PyArray_ConvertMultiAxis(PyObject *axis_in, int ndim, npy_bool *out_axis_flags)
 
         memset(out_axis_flags, 0, ndim);
 
-        axis = PyArray_PyIntAsInt(axis_in);
+        axis = PyArray_PyIntAsInt_ErrMsg(axis_in,
+                                   "an integer is required for the axis");
         axis_orig = axis;
 
         if (error_converting(axis)) {
@@ -395,6 +405,9 @@ PyArray_SortkindConverter(PyObject *obj, NPY_SORTKIND *sortkind)
 
     if (PyUnicode_Check(obj)) {
         obj = tmp = PyUnicode_AsASCIIString(obj);
+        if (obj == NULL) {
+            return NPY_FAIL;
+        }
     }
 
     *sortkind = NPY_QUICKSORT;
@@ -440,6 +453,9 @@ PyArray_SelectkindConverter(PyObject *obj, NPY_SELECTKIND *selectkind)
 
     if (PyUnicode_Check(obj)) {
         obj = tmp = PyUnicode_AsASCIIString(obj);
+        if (obj == NULL) {
+            return NPY_FAIL;
+        }
     }
 
     *selectkind = NPY_INTROSELECT;
@@ -526,6 +542,15 @@ PyArray_OrderConverter(PyObject *object, NPY_ORDER *val)
         return ret;
     }
     else if (!PyBytes_Check(object) || PyBytes_GET_SIZE(object) < 1) {
+        /* 2015-12-14, 1.11 */
+        int ret = DEPRECATE("Non-string object detected for "
+                            "the array ordering. Please pass "
+                            "in 'C', 'F', 'A', or 'K' instead");
+
+        if (ret < 0) {
+            return -1;
+        }
+
         if (PyObject_IsTrue(object)) {
             *val = NPY_FORTRANORDER;
         }
@@ -539,6 +564,18 @@ PyArray_OrderConverter(PyObject *object, NPY_ORDER *val)
     }
     else {
         str = PyBytes_AS_STRING(object);
+        if (strlen(str) != 1) {
+            /* 2015-12-14, 1.11 */
+            int ret = DEPRECATE("Non length-one string passed "
+                                "in for the array ordering. "
+                                "Please pass in 'C', 'F', 'A', "
+                                "or 'K' instead");
+
+            if (ret < 0) {
+                return -1;
+            }
+        }
+
         if (str[0] == 'C' || str[0] == 'c') {
             *val = NPY_CORDER;
         }
@@ -591,6 +628,9 @@ PyArray_ClipmodeConverter(PyObject *object, NPY_CLIPMODE *val)
         PyObject *tmp;
         int ret;
         tmp = PyUnicode_AsASCIIString(object);
+        if (tmp == NULL) {
+            return NPY_FAIL;
+        }
         ret = PyArray_ClipmodeConverter(tmp, val);
         Py_DECREF(tmp);
         return ret;
@@ -680,7 +720,7 @@ PyArray_CastingConverter(PyObject *obj, NPY_CASTING *casting)
         return ret;
     }
 
-    if (PyBytes_AsStringAndSize(obj, &str, &length) == -1) {
+    if (PyBytes_AsStringAndSize(obj, &str, &length) < 0) {
         return 0;
     }
 
@@ -727,13 +767,12 @@ PyArray_CastingConverter(PyObject *obj, NPY_CASTING *casting)
 * Other conversion functions
 *****************************/
 
-/*NUMPY_API*/
-NPY_NO_EXPORT int
-PyArray_PyIntAsInt(PyObject *o)
+static int
+PyArray_PyIntAsInt_ErrMsg(PyObject *o, const char * msg)
 {
     npy_intp long_value;
     /* This assumes that NPY_SIZEOF_INTP >= NPY_SIZEOF_INT */
-    long_value = PyArray_PyIntAsIntp(o);
+    long_value = PyArray_PyIntAsIntp_ErrMsg(o, msg);
 
 #if (NPY_SIZEOF_INTP > NPY_SIZEOF_INT)
     if ((long_value < INT_MIN) || (long_value > INT_MAX)) {
@@ -745,8 +784,14 @@ PyArray_PyIntAsInt(PyObject *o)
 }
 
 /*NUMPY_API*/
-NPY_NO_EXPORT npy_intp
-PyArray_PyIntAsIntp(PyObject *o)
+NPY_NO_EXPORT int
+PyArray_PyIntAsInt(PyObject *o)
+{
+    return PyArray_PyIntAsInt_ErrMsg(o, "an integer is required");
+}
+
+static npy_intp
+PyArray_PyIntAsIntp_ErrMsg(PyObject *o, const char * msg)
 {
 #if (NPY_SIZEOF_LONG < NPY_SIZEOF_INTP)
     long long long_value = -1;
@@ -754,7 +799,10 @@ PyArray_PyIntAsIntp(PyObject *o)
     long long_value = -1;
 #endif
     PyObject *obj, *err;
-    static char *msg = "an integer is required";
+    static PyObject *VisibleDeprecation = NULL;
+
+    npy_cache_import(
+        "numpy", "VisibleDeprecationWarning", &VisibleDeprecation);
 
     if (!o) {
         PyErr_SetString(PyExc_TypeError, msg);
@@ -763,8 +811,12 @@ PyArray_PyIntAsIntp(PyObject *o)
 
     /* Be a bit stricter and not allow bools, np.bool_ is handled later */
     if (PyBool_Check(o)) {
-        if (DEPRECATE("using a boolean instead of an integer"
-                      " will result in an error in the future") < 0) {
+        /* 2013-04-13, 1.8 */
+        if (PyErr_WarnEx(
+                VisibleDeprecation,
+                "using a boolean instead of an integer"
+                " will result in an error in the future",
+                1) < 0) {
             return -1;
         }
     }
@@ -796,8 +848,12 @@ PyArray_PyIntAsIntp(PyObject *o)
 
     /* Disallow numpy.bool_. Boolean arrays do not currently support index. */
     if (PyArray_IsScalar(o, Bool)) {
-        if (DEPRECATE("using a boolean instead of an integer"
-                      " will result in an error in the future") < 0) {
+        /* 2013-06-09, 1.8 */
+        if (PyErr_WarnEx(
+                VisibleDeprecation,
+                "using a boolean instead of an integer"
+                " will result in an error in the future",
+                1) < 0) {
             return -1;
         }
     }
@@ -863,8 +919,12 @@ PyArray_PyIntAsIntp(PyObject *o)
     }
     /* Give a deprecation warning, unless there was already an error */
     if (!error_converting(long_value)) {
-        if (DEPRECATE("using a non-integer number instead of an integer"
-                      " will result in an error in the future") < 0) {
+        /* 2013-04-13, 1.8 */
+        if (PyErr_WarnEx(
+                VisibleDeprecation,
+                "using a non-integer number instead of an integer"
+                " will result in an error in the future",
+                1) < 0) {
             return -1;
         }
     }
@@ -879,6 +939,7 @@ PyArray_PyIntAsIntp(PyObject *o)
         return -1;
     }
 
+    goto overflow_check; /* silence unused warning */
  overflow_check:
 #if (NPY_SIZEOF_LONG < NPY_SIZEOF_INTP)
   #if (NPY_SIZEOF_LONGLONG > NPY_SIZEOF_INTP)
@@ -900,6 +961,13 @@ PyArray_PyIntAsIntp(PyObject *o)
     return long_value;
 }
 
+/*NUMPY_API*/
+NPY_NO_EXPORT npy_intp
+PyArray_PyIntAsIntp(PyObject *o)
+{
+    return PyArray_PyIntAsIntp_ErrMsg(o, "an integer is required");
+}
+
 
 /*
  * PyArray_IntpFromIndexSequence
@@ -995,9 +1063,6 @@ NPY_NO_EXPORT int
 PyArray_TypestrConvert(int itemsize, int gentype)
 {
     int newtype = NPY_NOTYPE;
-    PyArray_Descr *temp;
-    const char *msg = "Specified size is invalid for this data type.\n"
-        "Size will be ignored in NumPy 1.7 but may throw an exception in future versions.";
 
     switch (gentype) {
         case NPY_GENBOOLLTR:
@@ -1114,6 +1179,7 @@ PyArray_TypestrConvert(int itemsize, int gentype)
             if (itemsize == 4 || itemsize == 8) {
                 int ret = 0;
                 if (evil_global_disable_warn_O4O8_flag) {
+                    /* 2012-02-04, 1.7, not sure when this can be removed */
                     ret = DEPRECATE("DType strings 'O4' and 'O8' are "
                             "deprecated because they are platform "
                             "specific. Use 'O' instead");
@@ -1151,31 +1217,6 @@ PyArray_TypestrConvert(int itemsize, int gentype)
             break;
     }
 
-    /*
-     * Raise deprecate warning if new type hasn't been
-     * set yet and size char is invalid.
-     * This should eventually be changed to an error in
-     * future NumPy versions.
-     */
-    if (newtype == NPY_NOTYPE) {
-        temp = PyArray_DescrFromType(gentype);
-        if (temp != NULL) {
-            if (temp->elsize != itemsize) {
-                if (DEPRECATE(msg) < 0) {
-                    Py_DECREF(temp);
-                    return -1;
-                }
-
-                newtype = gentype;
-            }
-            else {
-                newtype = gentype;
-            }
-
-            Py_DECREF(temp);
-        }
-    }
-
     return newtype;
 }
 
