@@ -160,6 +160,15 @@ scalar_value(PyObject *scalar, PyArray_Descr *descr)
 #undef _CHK
 }
 
+/*NUMPY_API
+ * return true an object is exactly a numpy scalar
+ */
+NPY_NO_EXPORT int
+PyArray_CheckAnyScalarExact(PyObject * obj)
+{
+    return is_anyscalar_exact(obj);
+}
+
 /*NUMPY_API
  * Convert to c-type
  *
@@ -634,7 +643,7 @@ PyArray_Scalar(void *data, PyArray_Descr *descr, PyObject *base)
     copyswap = descr->f->copyswap;
     type = descr->typeobj;
     swap = !PyArray_ISNBO(descr->byteorder);
-    if PyTypeNum_ISSTRING(type_num) {
+    if (PyTypeNum_ISSTRING(type_num)) {
         /* Eliminate NULL bytes */
         char *dptr = data;
 
@@ -831,8 +840,9 @@ PyArray_Scalar(void *data, PyArray_Descr *descr, PyObject *base)
 
 /*NUMPY_API
  *
- *Return either an array or the appropriate Python object if the array
- *is 0d and matches a Python type.
+ * Return either an array or the appropriate Python object if the array
+ * is 0d and matches a Python type.
+ * steals reference to mp
  */
 NPY_NO_EXPORT PyObject *
 PyArray_Return(PyArrayObject *mp)
