@@ -93,7 +93,8 @@ PyArray_Resize(PyArrayObject *self, PyArray_Dims *newshape, int refcheck,
                 || (PyArray_BASE(self) != NULL)
                 || (((PyArrayObject_fields *)self)->weakreflist != NULL)) {
             PyErr_SetString(PyExc_ValueError,
-                    "cannot resize an array references or is referenced\n"\
+                    "cannot resize an array that "\
+                    "references or is referenced\n"\
                     "by another array in this way.  Use the resize function");
             return NULL;
         }
@@ -153,8 +154,7 @@ PyArray_Resize(PyArrayObject *self, PyArray_Dims *newshape, int refcheck,
             PyArray_FLAGS(self), &(((PyArrayObject_fields *)self)->flags));
     memmove(PyArray_DIMS(self), new_dimensions, new_nd*sizeof(npy_intp));
     memmove(PyArray_STRIDES(self), new_strides, new_nd*sizeof(npy_intp));
-    Py_INCREF(Py_None);
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
 /*
@@ -325,8 +325,12 @@ _putzero(char *optr, PyObject *zero, PyArray_Descr *dtype)
         }
     }
     else {
-        Py_INCREF(zero);
-        NPY_COPY_PYOBJECT_PTR(optr, &zero);
+        npy_intp i;
+        for (i = 0; i < dtype->elsize / sizeof(zero); i++) {
+            Py_INCREF(zero);
+            NPY_COPY_PYOBJECT_PTR(optr, &zero);
+            optr += sizeof(zero);
+        }
     }
     return;
 }
@@ -608,7 +612,7 @@ PyArray_SqueezeSelected(PyArrayObject *self, npy_bool *axis_flags)
             else {
                 PyErr_SetString(PyExc_ValueError,
                         "cannot select an axis to squeeze out "
-                        "which has size greater than one");
+                        "which has size not equal to one");
                 return NULL;
             }
         }
@@ -648,19 +652,8 @@ PyArray_SwapAxes(PyArrayObject *ap, int a1, int a2)
 {
     PyArray_Dims new_axes;
     npy_intp dims[NPY_MAXDIMS];
-    int n, i, val;
-    PyObject *ret;
-
-    if (a1 == a2) {
-        Py_INCREF(ap);
-        return (PyObject *)ap;
-    }
-
-    n = PyArray_NDIM(ap);
-    if (n <= 1) {
-        Py_INCREF(ap);
-        return (PyObject *)ap;
-    }
+    int n = PyArray_NDIM(ap);
+    int i;
 
     if (a1 < 0) {
         a1 += n;
@@ -678,25 +671,20 @@ PyArray_SwapAxes(PyArrayObject *ap, int a1, int a2)
                         "bad axis2 argument to swapaxes");
         return NULL;
     }
+
+    for (i = 0; i < n; ++i) {
+        dims[i] = i;
+    }
+    dims[a1] = a2;
+    dims[a2] = a1;
+
     new_axes.ptr = dims;
     new_axes.len = n;
 
-    for (i = 0; i < n; i++) {
-        if (i == a1) {
-            val = a2;
-        }
-        else if (i == a2) {
-            val = a1;
-        }
-        else {
-            val = i;
-        }
-        new_axes.ptr[i] = val;
-    }
-    ret = PyArray_Transpose(ap, &new_axes);
-    return ret;
+    return PyArray_Transpose(ap, &new_axes);
 }
 
+
 /*NUMPY_API
  * Return Transpose.
  */
@@ -775,7 +763,8 @@ PyArray_Transpose(PyArrayObject *ap, PyArray_Dims *permute)
         PyArray_DIMS(ret)[i] = PyArray_DIMS(ap)[permutation[i]];
         PyArray_STRIDES(ret)[i] = PyArray_STRIDES(ap)[permutation[i]];
     }
-    PyArray_UpdateFlags(ret, NPY_ARRAY_C_CONTIGUOUS | NPY_ARRAY_F_CONTIGUOUS);
+    PyArray_UpdateFlags(ret, NPY_ARRAY_C_CONTIGUOUS | NPY_ARRAY_F_CONTIGUOUS |
+                        NPY_ARRAY_ALIGNED);
     return (PyObject *)ret;
 }
 
@@ -783,7 +772,7 @@ PyArray_Transpose(PyArrayObject *ap, PyArray_Dims *permute)
  * Sorts items so stride is descending, because C-order
  * is the default in the face of ambiguity.
  */
-int _npy_stride_sort_item_comparator(const void *a, const void *b)
+static int _npy_stride_sort_item_comparator(const void *a, const void *b)
 {
     npy_intp astride = ((const npy_stride_sort_item *)a)->stride,
             bstride = ((const npy_stride_sort_item *)b)->stride;
@@ -942,10 +931,8 @@ PyArray_Ravel(PyArrayObject *arr, NPY_ORDER order)
 
     newdim.ptr = val;
 
-    if (order == NPY_ANYORDER) {
-        order = PyArray_ISFORTRAN(arr) ? NPY_FORTRANORDER : NPY_CORDER;
-    }
-    else if (order == NPY_KEEPORDER) {
+    if (order == NPY_KEEPORDER) {
+        /* This handles some corner cases, such as 0-d arrays as well */
         if (PyArray_IS_C_CONTIGUOUS(arr)) {
             order = NPY_CORDER;
         }
@@ -953,6 +940,9 @@ PyArray_Ravel(PyArrayObject *arr, NPY_ORDER order)
             order = NPY_FORTRANORDER;
         }
     }
+    else if (order == NPY_ANYORDER) {
+        order = PyArray_ISFORTRAN(arr) ? NPY_FORTRANORDER : NPY_CORDER;
+    }
 
     if (order == NPY_CORDER && PyArray_IS_C_CONTIGUOUS(arr)) {
         return PyArray_Newshape(arr, &newdim, NPY_CORDER);
@@ -969,8 +959,14 @@ PyArray_Ravel(PyArrayObject *arr, NPY_ORDER order)
         PyArray_CreateSortedStridePerm(PyArray_NDIM(arr),
                                 PyArray_STRIDES(arr), strideperm);
 
-        stride = strideperm[ndim-1].stride;
+        /* The output array must be contiguous, so the first stride is fixed */
+        stride = PyArray_ITEMSIZE(arr);
+
         for (i = ndim-1; i >= 0; --i) {
+            if (PyArray_DIM(arr, strideperm[i].perm) == 1) {
+                /* A size one dimension does not matter */
+                continue;
+            }
             if (strideperm[i].stride != stride) {
                 break;
             }
@@ -981,7 +977,7 @@ PyArray_Ravel(PyArrayObject *arr, NPY_ORDER order)
         if (i < 0) {
             PyArrayObject *ret;
 
-            stride = strideperm[ndim-1].stride;
+            stride = PyArray_ITEMSIZE(arr);
             val[0] = PyArray_SIZE(arr);
 
             Py_INCREF(PyArray_DESCR(arr));
