@@ -1,18 +1,18 @@
 from __future__ import division, absolute_import, print_function
 
-from numpy.testing import *
-
 import numpy.distutils.fcompiler
+from numpy.testing import TestCase, run_module_suite, assert_
+
 
 intel_32bit_version_strings = [
-    ("Intel(R) Fortran Intel(R) 32-bit Compiler Professional for applications"\
+    ("Intel(R) Fortran Intel(R) 32-bit Compiler Professional for applications"
      "running on Intel(R) 32, Version 11.1", '11.1'),
 ]
 
 intel_64bit_version_strings = [
-    ("Intel(R) Fortran IA-64 Compiler Professional for applications"\
+    ("Intel(R) Fortran IA-64 Compiler Professional for applications"
      "running on IA-64, Version 11.0", '11.0'),
-    ("Intel(R) Fortran Intel(R) 64 Compiler Professional for applications"\
+    ("Intel(R) Fortran Intel(R) 64 Compiler Professional for applications"
      "running on Intel(R) 64, Version 11.1", '11.1')
 ]
 
